#include "test-ability.h"

#include <adventure-pack/ap-ability.h>

#include <glib.h>

static gboolean test_ability_signal_score_changed_ran = FALSE;
static gint test_ability_signal_score_changed_score = 0;

static gboolean test_ability_signal_modifier_changed_ran = FALSE;
static gint test_ability_signal_modifier_changed_score = 0;

void
ability_fixture_set_up (AbilityFixture *fixture,
                        gconstpointer   user_data)
{
    fixture->ability = ap_ability_new ();
}

void
ability_fixture_tear_down (AbilityFixture *fixture,
                           gconstpointer   user_data)
{
    g_clear_object (&fixture->ability);
}

void
test_ability_name (AbilityFixture *fixture,
                   gconstpointer   user_data)
{
    gchar *name;

    ap_ability_set_name (fixture->ability, "Example Ability");

    name = ap_ability_get_name (fixture->ability);

    g_assert_cmpstr (name, ==, "Example Ability");

    g_free (name);
}

void
test_ability_score (AbilityFixture *fixture,
                    gconstpointer   user_data)
{
    gint score;

    ap_ability_set_score (fixture->ability, 10);

    score = ap_ability_get_score (fixture->ability);

    g_assert_cmpint (score, ==, 10);
}

void
test_ability_score_with_bonus (AbilityFixture *fixture,
                               gconstpointer   user_data)
{
    gint score;

    ap_ability_set_score (fixture->ability, 10);
    ap_ability_set_bonus (fixture->ability, 3);

    score = ap_ability_get_score (fixture->ability);

    g_assert_cmpint (score, ==, 13);
}

void
test_ability_modifier (AbilityFixture *fixture,
                       gconstpointer   user_data)
{
    gint score;
    gint modifier;

    ap_ability_set_score (fixture->ability, GPOINTER_TO_INT (user_data));

    score = ap_ability_get_score (fixture->ability);
    modifier = ap_ability_get_modifier (fixture->ability);

    if (score == 1)
        g_assert_cmpint (modifier, ==, -5);
    else if (score == 2 || score == 3)
        g_assert_cmpint (modifier, ==, -4);
    else if (score == 4 || score == 5)
        g_assert_cmpint (modifier, ==, -3);
    else if (score == 6 || score == 7)
        g_assert_cmpint (modifier, ==, -2);
    else if (score == 8 || score == 9)
        g_assert_cmpint (modifier, ==, -1);
    else if (score == 10 || score == 11)
        g_assert_cmpint (modifier, ==, 0);
    else if (score == 12 || score == 13)
        g_assert_cmpint (modifier, ==, 1);
    else if (score == 14 || score == 15)
        g_assert_cmpint (modifier, ==, 2);
    else if (score == 16 || score == 17)
        g_assert_cmpint (modifier, ==, 3);
    else if (score == 18 || score == 19)
        g_assert_cmpint (modifier, ==, 4);
    else if (score == 20 || score == 21)
        g_assert_cmpint (modifier, ==, 5);
    else if (score == 22 || score == 23)
        g_assert_cmpint (modifier, ==, 6);
    else if (score == 24 || score == 25)
        g_assert_cmpint (modifier, ==, 7);
    else if (score == 26 || score == 27)
        g_assert_cmpint (modifier, ==, 8);
    else if (score == 28 || score == 29)
        g_assert_cmpint (modifier, ==, 9);
    else if (score == 30)
        g_assert_cmpint (modifier, ==, 10);
    else
        g_warning ("Score out of 1 - 30 range, is: \"%i\"", score);
}

static void
on_ability_signal_score_changed (GObject    *source,
                                 GParamSpec  pspec,
                                 gpointer    user_data)
{
    APAbility *ability = g_object_ref (AP_ABILITY (source));

    g_assert (AP_IS_ABILITY (ability));

    test_ability_signal_score_changed_ran = TRUE;
    test_ability_signal_score_changed_score = ap_ability_get_score (ability);

    g_object_unref (ability);
}

void
test_ability_signal_score_changed (AbilityFixture *fixture,
                                   gconstpointer   user_data)
{
    APAbility *ability = ap_ability_new ();

    g_signal_connect (ability,
                      "notify::score",
                      G_CALLBACK (on_ability_signal_score_changed),
                      NULL);

    ap_ability_set_score (ability, 10);

    g_assert (test_ability_signal_score_changed_ran);
    g_assert_cmpint (test_ability_signal_score_changed_score, ==, 10);

    g_object_unref (ability);
}

static void
on_ability_signal_modifier_changed (GObject    *source,
                                    GParamSpec  pspec,
                                    gpointer    user_data)
{
    APAbility *ability = g_object_ref (AP_ABILITY (source));

    g_assert (AP_IS_ABILITY (ability));

    test_ability_signal_modifier_changed_ran = TRUE;
    test_ability_signal_modifier_changed_score = ap_ability_get_modifier (ability);

    g_object_unref (ability);
}
void
test_ability_signal_modifier_changed (AbilityFixture *fixture,
                                      gconstpointer   user_data)
{
    APAbility *ability = ap_ability_new ();

    g_signal_connect (ability,
                      "notify::modifier",
                      G_CALLBACK (on_ability_signal_modifier_changed),
                      NULL);

    ap_ability_set_score (ability, 10);

    // Ensure modifier will change
    // ability score (10) = ability modifier (0)
    // ability score (12) = ability modifier (1)
    ap_ability_set_score (ability, 12);

    g_assert (test_ability_signal_modifier_changed_ran);
    g_assert_cmpint (test_ability_signal_modifier_changed_score, ==, 1);

    g_object_unref (ability);
}
